@@ -1,6 +1,6 @@
 package WWW::Google::Translate;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use strict;
 use warnings;
@@ -13,24 +13,26 @@ use warnings;
     use Readonly;
 }
 
-my ( $REST_HOST, $REST_URL, $CONSOLE_URL );
+my ( $REST_HOST, $REST_URL, $CONSOLE_URL, $GET_SIZE_LIMIT );
 {
-    Readonly $REST_HOST   => 'www.googleapis.com';
-    Readonly $REST_URL    => "https://$REST_HOST/language/translate/v2";
-    Readonly $CONSOLE_URL => "https://code.google.com/apis/console";
+    Readonly $REST_HOST      => 'www.googleapis.com';
+    Readonly $REST_URL       => "https://$REST_HOST/language/translate/v2";
+    Readonly $CONSOLE_URL    => "https://code.google.com/apis/console";
+    Readonly $GET_SIZE_LIMIT => 1894;    # trial & error (2K)
 }
 
 sub new {
     my ( $class, $param_rh ) = @_;
 
     my %self = (
-        key            => 0,
-        default_source => 'en',
-        default_target => 'es',
-        data_format    => 'perl',
-        timeout        => 60,
-        rest_url       => $REST_URL,
-        agent          => ( sprintf '%s/%s', __PACKAGE__, $VERSION ),
+        key              => 0,
+        default_source   => 'en',
+        default_target   => 'es',
+        data_format      => 'perl',
+        timeout          => 60,
+        transform_result => 1,
+        rest_url         => $REST_URL,
+        agent            => ( sprintf '%s/%s', __PACKAGE__, $VERSION ),
     );
 
     for my $property ( keys %self ) {
@@ -52,6 +54,8 @@ sub new {
     croak "data_format must either be Perl or JSON"
         if $self{data_format} !~ m{\A (?: perl|json ) \z}xmsi;
 
+    $self{transform_result} = $self{transform_result} ? 1 : 0;
+
     $self{ua} = LWP::UserAgent->new();
     $self{ua}->agent( delete $self{agent} );
 
@@ -74,7 +78,8 @@ sub translate {
         $self->{default_source} = $arg_rh->{source};
         $self->{default_target} = $arg_rh->{target};
 
-        my @unsupported = grep { $_ ne 'q' && $_ ne 'source' && $_ ne 'target' }
+        my @unsupported
+            = grep { $_ ne 'q' && $_ ne 'source' && $_ ne 'target' }
             keys %{$arg_rh};
 
         croak "unsupported parameters: ", ( join ',', @unsupported )
@@ -131,18 +136,34 @@ sub detect {
 sub _rest {
     my ( $self, $operation, $arg_rh ) = @_;
 
-    my $url = $self->{rest_url};
-
-    my $uri
-        = URI->new( $operation eq 'translate' ? $url : "$url/$operation" );
+    my $url
+        = $operation eq 'translate'
+        ? $self->{rest_url}
+        : $self->{rest_url} . "/$operation";
 
     my %form = (
         key => $self->{key},
         %{$arg_rh},
     );
-    $uri->query_form( \%form );
 
-    my $response = $self->{ua}->get($uri);
+    my $response;
+
+    if ( exists $form{q} && $GET_SIZE_LIMIT < length $form{q} ) {
+
+        $response = $self->{ua}->post(
+            $url,
+            'X-HTTP-Method-Override' => 'GET',
+            Content                  => \%form
+        );
+    }
+    else {
+
+        my $uri = URI->new($url);
+
+        $uri->query_form( \%form );
+
+        $response = $self->{ua}->get($uri);
+    }
 
     if ( $response->code() == HTTP_BAD_REQUEST ) {
 
@@ -163,11 +184,38 @@ sub _rest {
     }
 
     my $json = $response->content();
+    my $cache_control = $response->header('Cache-Control') || "";
 
-    return from_json($json)
-        if 'perl' eq lc $self->{data_format};
+    if (   $self->{transform_result}
+        && $cache_control !~ m{ no-transform }xmsi )
+    {
+        utf8::decode($json);
+    }
 
-    return $json;
+    return $json
+        if 'json' eq lc $self->{data_format};
+
+    my $trans_rh = from_json($json);
+
+    if (   $self->{transform_result}
+        && $cache_control !~ m{ no-transform }xmsi )
+    {
+        for my $i ( keys %{$trans_rh} ) {
+
+            for my $j ( keys %{ $trans_rh->{$i} } ) {
+
+                for my $k ( 0 .. $#{ $trans_rh->{$i}->{$j} } ) {
+
+                    for my $l ( keys %{ $trans_rh->{$i}->{$j}->[$k] } ) {
+
+                        utf8::encode( $trans_rh->{$i}->{$j}->[$k]->{$l} );
+                    }
+                }
+            }
+        }
+    }
+
+    return $trans_rh;
 }
 
 1;
